﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.IO;
using System.Windows.Forms;

namespace vnorene_seznamy
{
    public class Data
    {
        [Flags]
        public enum SaveTypes : byte
        {
            Synoptic = 0, //přehledná data
            Saving = 1 //šetřící data
        }

        private Dictionary<string, DataData> tree = new Dictionary<string, DataData>(); //celkový strom složek
        private List<string> lines = new List<string>(); //data souboru

        public Dictionary<string, DataData> Tree
        {
            get { return tree; }
        }
        public string ActualPath { get; set; }
        public bool IsSaved { get; set; }

        public Data(string path)
        {
            this.IsSaved = true;
            this.ActualPath = path;

            using (StreamReader sr = new StreamReader(path))
            {
                string line, acpa = null;
                int vnor = 0, index = 0;

                DataFolder fol = new DataFolder("", Path.GetFileName(path)); //aktuální složka
                List<DataFolder> lf = new List<DataFolder>(); //slořky pod aktuální složkou

                while ((line = sr.ReadLine()) != null)
                {
                    line = line.Trim();
                    lines.Add(line);

                    int isfo = line.IndexOf('-'); //folder
                    int isva = line.IndexOf(':'); //value
                    int isen = line.IndexOf('}'); //folder end

                    if (isfo > -1) //folder
                    {
                        //posunutí cesty
                        string name = line.Remove(isfo);
                        acpa += name + '/';

                        DataFolder folder = new DataFolder(acpa, name);
                        
                        folder.LineIndex = index;
                        folder.vnor = vnor + 1;

                        DataData d = new DataData(DataData.Type.Folder, folder);

                        fol.AddValue(d);

                        if (lf.Contains(fol))
                            lf[vnor] = fol;
                        else
                            lf.Add(fol);
                        
                        fol = folder;

                        //zvětšení vnořenosti
                        vnor++;
                    }
                    else if (isva > -1)  //value
                    {
                        int b = line.IndexOf('(');
                        string type;

                        //oprava hodnoty
                        if (b > -1) type = line.Substring(b + 1, line.Length - 2 - b); //datový typ
                        else type = "String";

                        string name = line.Substring(0, isva); //jméno
                        string value = line.Substring(isva + 1, b - isva - 1); //data

                        DataValue val = new DataValue(DataValue.GetDataTypeFromString(type), name, value);

                        val.LineIndex = index;

                        DataData d = new DataData(DataData.Type.Value, val);
                        fol.AddValue(d);
                        
                    }
                    else if (isen > -1) //end folder
                    {
                        //konec složky, vnořenost se sníží o jedno a změní se cesta
                        acpa = GetPath(acpa);
                        vnor--;

                        //vynulování a přepsání
                        lf[vnor].AddValue(new DataData(DataData.Type.Folder, fol));
                        fol = lf[vnor];

                        DataData d = new DataData(DataData.Type.Folder, lf[0]);
                        if (tree.ContainsKey(lf[0].Name))
                            tree[lf[0].Name] = d;
                        else
                            tree.Add(lf[0].Name, d);

                        lf.Remove(lf.Last());
                    }
                    index++;
                }
                lf.Clear();
            }
        }

        public void Save(string path, SaveTypes type)
        {
            IsSaved = true;
            ActualPath = path;
            int tabs = 0;

            using (StreamWriter sw = new StreamWriter(path))
            {
                foreach (string line in lines.ToArray())
                {
                    string write = line.Trim();

                    if (line == "}")
                        tabs--;

                    if (type == SaveTypes.Synoptic) //přehledný typ
                        write = get_tabs(tabs) + write; //přidání odsazení (tabulátorů)

                    sw.WriteLine(write);

                    if (line == "{")
                        tabs++;
                }
                sw.Flush(); sw.Close();
            }
        }

        private string get_tabs(int tabs)
        {
            string tab = "";
            for (int i = 0; i < tabs; i++)
            {
                tab = tab + '\t';
            }
            return tab;
        }

        public void SaveAsBinaryFile(string path)
        { 
            //zatím nevyřešeno
        }

        public void RenameValue(DataValue what, string new_name)
        {
            int index = what.LineIndex;
            lines[index] = new_name + ':' + what.Data + '(' + what.DataType.ToString() + ')';
            this.IsSaved = false;
        }

        public void RenameFolder(DataFolder what, string new_name)
        {
            int index = what.LineIndex;
            lines[index] = new_name + '-';
            this.IsSaved = false;
        }

        public void AddValue(DataFolder where, DataValue value)
        {
            int index = where.LineIndex;
            string tab = where.Tabs;
            lines.Insert(index + 2, value.Name + ':' + value.Data + '(' + value.DataType.ToString() + ')');
            this.IsSaved = false;
        }

        public void AddFolder(DataFolder where, string folder_name)
        {
            int index = where.LineIndex;

            lines.Insert(index + 2, folder_name + '-');
            lines.Insert(index + 3, "{");
            lines.Insert(index + 4, "}");

            this.IsSaved = false;
        }

        public void SetValue(DataValue val)
        {
            lines[val.LineIndex] = val.Name + ':' + val.Data + '(' + val.DataType.ToString() + ')';
            this.IsSaved = false;
        }

        private string GetPath(string path)
        {
            char c = ' ';
            int i = path.Length - 2;

            while (c != '/' && i > 0)
            {
                c = char.Parse(path.Substring(i, 1));
                i--;
            }
            return path.Substring(0, i + 2);
        }
    }

    public class DataFolder
    {
        public Dictionary<string, DataData> tree = new Dictionary<string, DataData>(); //strom složek této složky

        public string Name { get; set; } //jméno
        public string Path { get; set; } //cesta
        public int LineIndex { get; set; } //pozice složky v souboru

        public int vnor = 0;

        public string Tabs 
        {
            get
            {
                string tab = "";
                for (int i = 0; i < vnor; i++)
                {
                    tab += '\t';
                }
                return tab;
            }
        }

        public DataFolder(string path, string name)
        {
            this.Path = path;
            this.Name = name; 
        }

        public void AddValue(DataData value)
        {
            if (value.T == DataData.Type.Folder) //přidání složky
            {
                if (!tree.ContainsKey(((DataFolder)value.Data).Name))
                    tree.Add(((DataFolder)value.Data).Name, value); //přidání
                else
                    tree[((DataFolder)value.Data).Name] = value; //přepsání
            }
            else if (value.T == DataData.Type.Value) //přidání hodnoty
            {
                if (!tree.ContainsKey(((DataValue)value.Data).Name))
                    tree.Add(((DataValue)value.Data).Name, value); //přidání
                else
                    tree[((DataValue)value.Data).Name] = value; //přepsání
            }
        }        
    }

    public class DataData
    { 
        [Flags]
        public enum Type : byte
        {
            Folder = 0, Value = 1 
        }

        public object Data { get; set; }
        public Type T { get; set; }

        public DataData(Type t, object data)
        {
            this.T = t;
            this.Data = data;
        }
    }

    public class DataValue
    {
        [Flags]
        public enum DataTypes : byte
        { 
          //0 - 1     0 - 255   znak           číslo        text        des. číslo   datum     PSČ
            Bool = 0, Byte = 1, Character = 2, Integer = 3, String = 4, Decimal = 5, Date = 6, ZIP = 7
        }

        public string Name { get; set; }
        public string Data { get; set; }
        public DataTypes DataType { get; set; }
        public int LineIndex { get; set; }
 
        public DataValue(DataTypes type, string name, string data)
        {
            this.Name = name;
            this.Data = data;
            this.DataType = type;
        }

        public static DataTypes GetDataTypeFromString(string data)
        {
            Dictionary<string, int> types = new Dictionary<string,int>()
            { 
                { "Bool", 0 }, 
                { "Byte", 1 }, 
                { "Character", 2 },
                { "Integer", 3 }, 
                { "String", 4 },
                { "Decimal", 5 }, 
                { "Date", 6 },
                { "ZIP", 7 }
            };

            if (types.ContainsKey(data))
                return (DataTypes)types[data];
            else
                return DataTypes.String;
        }
    }
}
