﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using System.IO;

namespace vnorene_seznamy
{
    public partial class Form1 : Form
    {
        private Data data; //data souboru
        private Dictionary<string, DataFolder> folders = new Dictionary<string, DataFolder>(); //složky

        private string actual_node; //aktuální název složky
        private Configuration config; //nastavení

        public Form1(Configuration config)
        {
            this.config = config;
            InitializeComponent();

            if (config.SaveType == Data.SaveTypes.Saving)
                this.šetřitMístoToolStripMenuItem.Enabled = false;
            else
                this.přehledněToolStripMenuItem.Enabled = false;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            set_values_enabled(false);
        }

        private void load(string p)
        {
            clear();

            treeView1.Nodes.Add(p);

            //načte všechny složky a podsložky 
            addall(data.Tree, treeView1.Nodes[0]);
        }

        private void addall(Dictionary<string, DataData> datas, TreeNode w)
        {
            for (int i = 0; i < datas.Count; i++)
            { 
                DataData d = datas[datas.Keys.ToArray()[i]];
                if (d.T == DataData.Type.Folder)
                {
                    DataFolder folder = (DataFolder)d.Data;
                    TreeNode node = new TreeNode(folder.Name);

                    node.Name = folder.Path;
                    folders.Add(folder.Path, folder);

                    addall(folder.tree, node);
                    w.Nodes.Add(node);                    
                }
            }
        }

        private void treeView1_select(object sender, TreeViewEventArgs e)
        {
            listView1.Items.Clear();
            if (folders.ContainsKey(e.Node.Name))
            {
                DataFolder f = folders[e.Node.Name];
                actual_node = e.Node.Name;

                List<string> names = new List<string>();
                List<DataValue> values = new List<DataValue>();

                foreach (DataData d in f.tree.Values.ToArray())
                {
                    if (d != null)
                    {
                        if (d.T == DataData.Type.Value)
                        {
                            values.Add((DataValue)d.Data);
                            names.Add(((DataValue)d.Data).Name);
                        }
                    }
                }

                Array.Sort(names.ToArray(), values.ToArray());

                foreach (DataValue val in values)
                {
                    ListViewItem item = new ListViewItem(val.Name);
                    item.SubItems.Add(val.DataType.ToString());
                    item.SubItems.Add(val.Data);
                    listView1.Items.Add(item);
                }
            }

            set_folders_enabled(true);
        }

        private void otevřítToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Filter = "Databáze vnořených seznamů|*.dvs|.bin|*.bin";
            ofd.Multiselect = false;
            ofd.Title = "Otevřete soubor s vnořenými seznami";
            if (ofd.ShowDialog() == DialogResult.OK)
            {
                data = new Data(ofd.FileName); //načíst data
                load(Path.GetDirectoryName(ofd.FileName)); //zobrazit data
            }
        }

        private void treeView1_AfterLabelEdit(object sender, NodeLabelEditEventArgs e)
        {
            string name = e.Label;
            if (name != null && name != "")
            {
                DataFolder folder = folders[e.Node.Name];

                string[] p = folder.Path.Split('/');
                string path = "";

                for (int i = 0; i < p.Length - 2; i++)
                { 
                    path += p[i] + '/';
                }


                folders.Remove(e.Node.Name);
                path += name + '/';
                folder.Path = path;

                folders.Add(folder.Path, folder);
                data.RenameFolder(folder, name);
            }
            else
            {
                e.CancelEdit = true;
            }
        }

        private void listView1_AfterLabelEdit(object sender, LabelEditEventArgs e)
        {
            string name = e.Label;
            if (name != null && name != "")
            {
                DataFolder folder = folders[actual_node];
                ListViewItem item = listView1.Items[e.Item];

                DataValue val = (DataValue)folder.tree[item.Text].Data;
                val.Name = name;
                data.RenameValue(val, name);
                
                folders[actual_node].tree.Remove(item.Text);
                folders[actual_node].tree.Add(name, new DataData(DataData.Type.Value, val));
            }
            else
            {
                e.CancelEdit = true;
            }
        }

        private void uložitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            data.Save(data.ActualPath, config.SaveType);
        }

        private void new_value(DataValue.DataTypes type)
        {
            DataValue val = new DataValue(type, "nová hodnota", "");
            DataFolder fol = folders[actual_node];

            val.LineIndex = fol.LineIndex + 2;
            
            data.AddValue(fol, val);

            folders[actual_node].AddValue(new DataData(DataData.Type.Value, val));

            ListViewItem item = new ListViewItem("nová hodnota");

            item.SubItems.Add(type.ToString());
            item.SubItems.Add(val.Data);

            listView1.Items.Insert(0, item);
            listView1.Items[0].BeginEdit();

            data.IsSaved = false;
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (data != null)
            {
                if (!data.IsSaved)
                {
                    DialogResult dialog = MessageBox.Show("Soubor nebyl, uložen, chcete uložit změny před ukončením programu?", "Uložit změny?", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Warning);
                    if (dialog == DialogResult.Yes)
                        data.Save(data.ActualPath, Data.SaveTypes.Synoptic);
                    else if (dialog == DialogResult.Cancel)
                        e.Cancel = true;
                }
            }
        }

        private void konecToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Dispose();
        }

        private void boolToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem obj = (ToolStripMenuItem)sender;
            DataValue.DataTypes type = DataValue.GetDataTypeFromString(obj.Text);
            new_value(type);
        }

        private void zavřítToolStripMenuItem_Click(object sender, EventArgs e)
        {
            data = null;
            clear();
        }

        private void clear()
        {
            treeView1.Nodes.Clear();
            listView1.Items.Clear();
            folders.Clear();
        }

        private void šetřitMístoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            config.SaveType = Data.SaveTypes.Saving;
            šetřitMístoToolStripMenuItem.Enabled = false;
            přehledněToolStripMenuItem.Enabled = true;
        }

        private void přehledněToolStripMenuItem_Click(object sender, EventArgs e)
        {
            config.SaveType = Data.SaveTypes.Synoptic;
            šetřitMístoToolStripMenuItem.Enabled = true;
            přehledněToolStripMenuItem.Enabled = false;
        }

        private void nováSložkaToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DataFolder fol = folders[actual_node];

            string plus = "";
            while (folders.ContainsKey(fol.Path + "nová složka" + plus)) //pokud složka s tímto názvem už existuje
            {
                int s = 0;
                bool tp = int.TryParse(plus, out s);

                if (tp) s += 1; //přidá se za název číslo

                plus = s.ToString();                
            }
            
            DataFolder folder = new DataFolder(fol.Path + "nová složka" + plus, "nová složka" + plus);

            folder.LineIndex = fol.LineIndex + 2;

            TreeNode node = new TreeNode(folder.Name);

            node.Name = folder.Path;
            folders.Add(folder.Path, folder);

            data.AddFolder(fol, folder.Name);

            treeView1.SelectedNode.Nodes.Insert(0, node);
            treeView1.SelectedNode.Nodes[0].BeginEdit();

            data.IsSaved = false;
        }

        private void uložitJakoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SaveFileDialog sfd = new SaveFileDialog();
            sfd.Filter = "Databáze vnořených seznamů|*.dvs|.bin|*.bin";
            if (sfd.ShowDialog() == DialogResult.OK)
            {
                data.Save(sfd.FileName, config.SaveType);
            }
        }

        private void odstranitHodonotuToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DataFolder fol = folders[actual_node];
            
                        
        }

        private void listView1_Click(object sender, EventArgs e)
        {
        }

        private void set_values_enabled(bool to)
        {
            ToolStripMenuItem[] controls = { změnitHodnotuToolStripMenuItem, odstranitHodonotuToolStripMenuItem };
            foreach (ToolStripMenuItem item in controls)
            {
                item.Enabled = to;
            }
        }

        private void set_folders_enabled(bool to)
        {
            ToolStripMenuItem[] controls = { nováHodnotaToolStripMenuItem, nováSložkaToolStripMenuItem, odstranitSložkuToolStripMenuItem };
            foreach (ToolStripMenuItem item in controls)
            {
                item.Enabled = to;
            }
        }

        private void listView1_SelectedIndexChanged(object sender, EventArgs e)
        {
            set_values_enabled(false);
            if (listView1.SelectedItems.Count > 0)
            {
                set_values_enabled(true);
            }
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            //uložit nastavení
            
        }

        private void změnitHodnotuToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ListViewItem item = listView1.SelectedItems[0];
            DataFolder folder = folders[actual_node];
            DataValue val = (DataValue)folder.tree[item.Text].Data;
            edit_value(val);
        }

        private void edit_value(DataValue val)
        {
            Edit e = new Edit(val.DataType, val);
            if (e.ShowDialog() == DialogResult.OK)
            {
                DataValue new_val = new DataValue(val.DataType, e.new_name, e.new_data);
                new_val.LineIndex = val.LineIndex;
                data.SetValue(new_val);
            }
        }

        private void listView1_DoubleClick(object sender, EventArgs e)
        {
            if (listView1.SelectedItems.Count > 0)
            {
                ListViewItem item = listView1.SelectedItems[0];
                DataFolder folder = folders[actual_node];
                DataValue val = (DataValue)folder.tree[item.Text].Data;
                edit_value(val);
            }
        }
    }
}
